/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.wsdl.wsdl20.xerces;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import javax.xml.namespace.QName;

import org.apache.xerces.dom.DOMMessageFormatter;
import org.bluestemsoftware.specification.eoa.component.RootComponent.ComponentName;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.InterfaceElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.InterfaceFaultElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.InterfaceOperationElement;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class InterfaceElementImpl extends AbstractWSDLElement implements InterfaceElement {

    private static final long serialVersionUID = 1L;

    private boolean isRead;
    private Map<String, InterfaceFaultElement> faultElements = new HashMap<String, InterfaceFaultElement>();
    private Map<String, InterfaceOperationElement> operationElements = new HashMap<String, InterfaceOperationElement>();

    public InterfaceElementImpl(WSDL20DocumentImpl owner) {
        super(owner, NAME.getNamespaceURI(), NAME.getPrefix() + ":" + NAME.getLocalPart());
    }

    public InterfaceElementImpl() {
        super();
    }
    
    public boolean isRead() {
        return isRead;
    }

    public void setIsRead(boolean isRead) {
        this.isRead = isRead;
    }

    public void setName(String name) {
        setAttribute("name", name);
    }

    public String getName() {
        return getAttribute("name").equals("") ? null : getAttribute("name");
    }

    public void setExtends(Set<QName> values) {
        if (values == null) {
            return;
        }
        String delim = "";
        StringBuffer sb = new StringBuffer();
        for (QName value : values) {
            String prefix = ((WSDL20DocumentImpl)getOwnerDocument()).addNamespace(value.getNamespaceURI());
            sb.append(delim + prefix + ":" + value.getLocalPart());
            delim = " ";
        }
        setAttribute("extends", sb.toString());
    }

    public Set<QName> getExtends() {
        StringTokenizer st = new StringTokenizer(getAttribute("extends"), " ", false);
        Set<QName> values = new HashSet<QName>();
        while (st.hasMoreTokens()) {
            values.add(ComponentName.valueOf(this, st.nextToken()));
        }
        return values;
    }

    public void setStyleDefaults(Set<String> styles) {
        if (styles == null) {
            return;
        }
        StringBuilder sb = new StringBuilder("");
        String delim = "";
        for (String style : styles) {
            sb.append(delim);
            sb.append(style);
            delim = " ";
        }
        setAttribute("styleDefault", sb.toString());
    }

    public Set<String> getStyleDefaults() {
        Set<String> styleDefaults = new HashSet<String>();
        StringTokenizer st = new StringTokenizer(getAttribute("styleDefault"));
        while (st.hasMoreTokens()) {
            styleDefaults.add(st.nextToken());
        }
        return styleDefaults;
    }

    public InterfaceFaultElement addFaultElement(InterfaceFaultElement faultElement) {
        if (faultElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        if (operationElements.size() > 0) {
            super.insertBefore(faultElement, operationElements.values().iterator().next());
        } else {
            super.insertBefore(faultElement, null);
        }
        faultElements.put(faultElement.getName(), faultElement);
        return faultElement;
    }

    public Map<String, InterfaceFaultElement> getFaultElements() {
        return faultElements;
    }

    public InterfaceOperationElement addOperationElement(InterfaceOperationElement operationElement) {
        if (operationElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        super.insertBefore(operationElement, null);
        operationElements.put(operationElement.getName(), operationElement);
        return operationElement;
    }

    public Map<String, InterfaceOperationElement> getOperationElements() {
        return operationElements;
    }

    @Override
    public Node appendChild(Node newChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(InterfaceFaultElement.NAME)) {
                return addFaultElement((InterfaceFaultElement)newChild);
            }
            if (elementName.equals(InterfaceOperationElement.NAME)) {
                return addOperationElement((InterfaceOperationElement)newChild);
            }
        }
        return super.insertBefore(newChild, null);
    }

    @Override
    public Node removeChild(Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(InterfaceFaultElement.NAME)) {
                faultElements.remove(((Element)oldChild).getAttribute("name"));
            }
            if (elementName.equals(InterfaceOperationElement.NAME)) {
                operationElements.remove(((Element)oldChild).getAttribute("name"));
            }
        }
        return super.removeChild(oldChild);
    }

    @Override
    public Node replaceChild(Node newChild, Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(InterfaceFaultElement.NAME)
                    || elementName.equals(InterfaceOperationElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.replaceChild(newChild, oldChild);
    }

    @Override
    public Node insertBefore(Node newChild, Node refChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(InterfaceFaultElement.NAME)
                    || elementName.equals(InterfaceOperationElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.insertBefore(newChild, refChild);
    }
    
}
